'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _tv = require('tv4');

var _tv2 = _interopRequireDefault(_tv);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var allowNull = function allowNull(data, schema) {
  return data === null && (schema.type === 'null' || schema.type.includes('null'));
};

_tv2.default.addFormat('objectid', function (data, schema) {
  if (allowNull(data, schema)) return null;
  if (typeof data === 'string' && /^[a-f\d]{24}$/i.test(data)) return null;
  return 'objectid expected';
});

_tv2.default.addFormat('date-time', function (data, schema) {
  if (allowNull(data, schema)) return null;
  // Source: http://stackoverflow.com/questions/3143070/javascript-regex-iso-datetime
  var dateTimeRegex = /\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+([+-][0-2]\d:[0-5]\d|Z)/;
  if (typeof data === 'string' && dateTimeRegex.test(data)) return null;
  return 'date-time, ISOString format, expected';
});

_tv2.default.addFormat('date', function (data, schema) {
  if (allowNull(data, schema)) return null;
  if (typeof data === 'string' && /\d{4}-[01]\d-[0-3]\d/.test(data)) return null;
  return 'date, YYYY-MM-DD format, expected';
});

_tv2.default.addFormat('time', function (data, schema) {
  if (allowNull(data, schema)) return null;
  if (typeof data === 'string' && /^[0-2][0-9]:[0-5][0-9](:[0-5][0-9])?$/.test(data)) return null;
  return 'time, HH:mm:ss or HH:mm format, expected';
});

_tv2.default.addFormat('email', function (data, schema) {
  if (allowNull(data, schema)) return null;
  // Source: http://stackoverflow.com/questions/46155/validate-email-address-in-javascript
  var emailRegex = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/;
  if (typeof data === 'string' && emailRegex.test(data)) return null;
  return 'email expected';
});

_tv2.default.addFormat('non-negative-integer', function (data, schema) {
  if (allowNull(data, schema)) return null;
  if (typeof data === 'string' && /^[0-9]+$/.test(data)) return null;
  return 'non negative integer expected';
});

module.exports = _tv2.default;
module.exports.assertValid = function (data, schema, errorMessage) {
  (0, _assert2.default)(data !== undefined, 'data must be defined');
  (0, _assert2.default)((typeof schema === 'undefined' ? 'undefined' : _typeof(schema)) === 'object', 'schema must be an object');
  if (errorMessage != null) (0, _assert2.default)(typeof errorMessage === 'string', 'errorMessage must be a string');
  var cleanedData = JSON.parse(JSON.stringify(data)); // remove undefined, convert dates to ISO strings, etc

  var _tv4$validateResult = _tv2.default.validateResult(cleanedData, schema, null, true),
      valid = _tv4$validateResult.valid,
      error = _tv4$validateResult.error;

  if (!valid) {
    var message = function () {
      var result = '';
      if (errorMessage) result += errorMessage + '; ';
      result += 'failed';
      if (schema.title) result += ' "' + schema.title + '"';
      result += ' schema validation';
      if (error.schemaPath.length) result += ' at schema path ' + error.schemaPath;
      if (error.dataPath.length) result += ' for data path ' + error.dataPath;
      result += '; ' + error.message.toLowerCase();
      return result;
    }();
    error.name = 'SchemaValidationError';
    error.message = message;
    error.data = data;
    error.schema = schema;
    throw error;
  }
};